/**
 * Peak determination by function approximation, using a multilayer perceptron.
 * 
 * Usage:
 * 
 * ./mlp plot limit load_file save_file  < input_spectrum
 */

#include <doublefann.h>
#include <string.h>
#include "common.cpp"

int main(int argc, char* argv[])
{
    double limit;
    char plot = 0, *save_file, *load_file;
    
    /* read the program options */
    if(argc >= 2) plot      = argv[1][0] == 'p';
    if(argc >= 3) limit     = atof(argv[2]); else limit = 0.8;
    if(argc >= 4) load_file = argv[3]; else load_file = NULL;
    if(argc >= 5) save_file = argv[4]; else save_file = NULL;
    if(load_file && strcmp(load_file, "null") == 0 ) load_file = NULL;
    if(save_file && strcmp(save_file, "null") == 0 ) save_file = NULL;
   
    /* read and scale the input data */
    Data data;
    data.read(limit);
    
    double min = data.min_x, max = data.max_x;
    data.scale_x(0, 1);
    
    /* create a neural network or load a existing one from a file */
    struct fann *ann;
    int n_epochs;
    
    if(load_file)
    {
        ann = fann_create_from_file(load_file);
        n_epochs = 1e3;
    }
    else
    {
        ann = fann_create_standard(3, 1, 3, 1);
        fann_randomize_weights(ann, -0.5, 0.5);
        
        fann_set_activation_function_hidden(ann, FANN_SIGMOID);
        fann_set_activation_function_output(ann, FANN_SIGMOID);
        
        fann_set_training_algorithm(ann, FANN_TRAIN_RPROP);
        n_epochs = 2e4;
    }
    
    if(!ann)
    {
        fprintf(stderr, "Couldn't create the neural network\n");
        return -1;
    }
    
    /* train for a fixed number of steps */
    int epoch;
    for(epoch = 0; epoch < n_epochs; ++epoch)
    {
        for(unsigned i = 0, e = data.size(); i < e; ++i)
            fann_train(ann, &data[i].x, &data[i].y);
    }
    
    /* find the peak by using the maximum algorithm */
    double peak_x = -1, peak_y = -1;
    double step_size = 1e-4;
    int n_steps      = (max - min) / step_size;
    
    for(int i = 0; i <= n_steps; ++i)
    {
        double w  = min + i * step_size;
        double x  = scale(w, min, max, 0, 1);
        double* o = fann_run(ann, &x);
        if(o[0] > peak_y) { peak_y = o[0]; peak_x = w;};
        if(plot) printf("%.12lf %.12lf\n", w, o[0]);
    }
    
    if(!plot) printf("%lf (%lf)\n", peak_x, fann_get_MSE(ann));
    
    /* save the network for future usage? */
    if(save_file) fann_save(ann, save_file);
    
    fann_destroy(ann);
    
    return 0;
}
