/**
 * Tries to fit the input spectrum with one or more gaussians. The gaussians
 * pararameters (weight, center and variation) are optimized with a particle 
 * swarm optimization technic.
 * 
 * Usage:
 * 
 * ./pso_gaussian plot limit < input_spectrum 
 */

#include "common.cpp"

extern "C" {
    #include <cpso.h>
}

static Data data;

static double gaussian(double x, double a, double c, double d)
{
    double t = (c - x);
    return a * exp( -t*t / (2*d*d) );
}

/* cost function */
static double eval(CpsoState* st, const double* p)
{
    double err = 0;
    
    for(unsigned i = 0, e = data.size(); i < e; ++i)
    {
        double r = gaussian(data[i].x, p[0], p[1], p[2]);
        double d = data[i].y - r;
        err     += d * d;
    }
    
    return -err / data.size(); 
}

int main(int argc, char* argv[])
{
    /* read the program options */
    char plot = 0;
    double limit;
    if(argc >= 2) plot  = argv[1][0] == 'p';
    if(argc >= 3) limit = atof(argv[2]); else limit = 0.4;
    
    /* read the input data */
    data.read(limit);
    double min = data.min_x, max = data.max_x;
    
    /* configure the PSO */
    CpsoState* st = cpso_new(3, 35);
    
    cpso_set_limits(st, 0, 0.5, 1);
    cpso_set_limits(st, 1, min, max);
    cpso_set_limits(st, 2, 0, max - min);
    
    cpso_set_eval  (st, &eval, NULL);
    cpso_set_params(st, 1.85, 1.5, 0, 0.01);
        
    random_seed();
    cpso_init(st);
        
    /* optimize */
    cpso_run(st, 200, 0);
    
    /* get the peak using the maximum algorithm */
    const double* best;
    double fit = cpso_get_best(st, -1, &best);
    if(!plot) printf("%lf (%lf)\n", best[1], -fit);
    
    if(plot)
    {
        for(double w = min; w <= max; w += 1e-3)
        {
            double y = gaussian(w, best[0], best[1], best[2]);
            printf("%.12lf %.12lf\n", w, y);
        }
    }
    
    cpso_free(st);
    
    return 0;
}
