/**
 * Tries to fit the input spectrum with a Nth-order polynomial. The polynomial
 * coeficients are optimized with a particle swarm optimization technic.
 * 
 * Usage:
 * 
 * ./pso_poly plot limit < input_spectrum 
 */

#include "common.cpp"

extern "C" {
    #include <cpso.h>
}

const int ORDER = 3 + 1;
static Data data;

/* polynomial */
double calc(const double* c, double x)
{
    double sum = 0;
    
    for(int i = 0; i < ORDER; ++i)
        sum += c[i] * pow(x, i);
    
    return sum;
}

/* cost function */
static double eval(CpsoState* st, const double* p)
{
    double mse = 0;
    
    for(unsigned i = 0, e = data.size(); i < e; ++i)
    {
        double err = data[i].y - calc(p, data[i].x);
        mse += err * err;
    }
    
    return -mse / data.size();
}

int main(int argc, char* argv[])
{
    /* read the program options */
    char plot = 0;
    double limit;
    if(argc >= 2) plot  = argv[1][0] == 'p';
    if(argc >= 3) limit = atof(argv[2]); else limit = 0.8;
    
    /* read and scale the input data */
    data.read(limit);
    
    double min = data.min_x, max = data.max_x;
    data.scale_x(0, 1);
    
    /* configure the PSO */
    CpsoState* st = cpso_new(ORDER, 35);
    
    for(int i = 0; i < ORDER; ++i)
        cpso_set_limits(st, i, -1, 1);
    
    cpso_set_eval  (st, &eval, NULL);
    cpso_set_params(st, 1.85, 1.55, 0.4, 0.015);
        
    random_seed();
    cpso_init(st);
    
    /* optimize */
    cpso_run(st, 1500, 0);
    
    /* find the peak by the maximum algorithm */
    const double* best;
    double fit = cpso_get_best(st, -1, &best);
    
    double peak_x = -1, peak_y = -1;
    double step_size = 1e-4;
    int n_steps      = (max - min) / step_size;
    
    for(int i = 0; i <= n_steps; ++i)
    {
        double w  = min + i * step_size;
        double x  = scale(w, min, max, 0, 1);
        double y = calc(best, x);
        if(y > peak_y) { peak_y = y; peak_x = w;};
        if(plot) printf("%.12lf %.12lf\n", w, y);
    }
    
    if(!plot) printf("%lf (%lf)\n", peak_x, -fit);
    cpso_free(st);
    
    return 0;
}
